@@ -349,7 +349,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
                     if (null == listeners.putIfAbsent(SimpleKey.valueOf(userId, contextId), listener)) {
                         listener.start();
                         unlock = false;
-                        LOGGER.info("Started IMAP-IDLE listener for user {} in context {} with session {}", I(userId), I(contextId), session.getSessionID());
+                        LOGGER.info("Started IMAP-IDLE listener for user {} in context {} with session {} ({})", I(userId), I(contextId), session.getSessionID(), session.getClient());
                         return listener;
                     }
 
@@ -367,7 +367,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
                 }
             }
         } else {
-            LOGGER.info("Could not acquire lock to start IMAP-IDLE listener for user {} in context {} with session {} as there is already an associated listener", I(userId), I(contextId), session.getSessionID());
+            LOGGER.info("Could not acquire lock to start IMAP-IDLE listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient());
         }
 
         // No listener registered for given session
@@ -492,7 +492,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
             // Query local ones first
             Collection<Session> sessions = sessiondService.getSessions(userId, contextId);
             for (Session session : sessions) {
-                if (!oldSessionId.equals(session.getSessionID()) && PushUtility.allowedClient(session.getClient())) {
+                if (!oldSessionId.equals(session.getSessionID()) && PushUtility.allowedClient(session.getClient(), session, true)) {
                     return injectAnotherListenerUsing(session, false).injectedPushListener;
                 }
             }
@@ -502,7 +502,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
                 // Look-up remote sessions, too, if possible
                 sessions = lookUpRemoteSessionsFor(userId, contextId);
                 for (Session session : sessions) {
-                    if (!oldSessionId.equals(session.getSessionID()) && PushUtility.allowedClient(session.getClient())) {
+                    if (!oldSessionId.equals(session.getSessionID()) && PushUtility.allowedClient(session.getClient(), session, true)) {
                         return injectAnotherListenerUsing(session, false).injectedPushListener;
                     }
                 }
